//ICC-AVR application builder : 04.03.2009 13:18:59
// Target : M8
// Crystal: 14.7456Mhz

#include <iom8v.h>
#include <macros.h>


#define int10flag  	  		  	0x01
#define AlarmSection1  		  	0x02
#define AlarmSection2  		  	0x04
#define AlarmSection3  		  	0x08
#define SendFrame				0x10


#define Section3Down  		  	0x04
#define Section3Up  		  	0x08
#define Section2Down  		  	0x10
#define Section2Up  		  	0x20
#define Section1Down  		  	0x40
#define Section1Up  		  	0x80


#define Ponton4Section1		  	0x01
#define Ponton4Section2		  	0x02
#define Ponton4Section3		  	0x04

#define Ponton3Section1		  	0x08
#define Ponton3Section2		  	0x10
#define Ponton3Section3		  	0x20

#define Ponton2Section1		  	0x40
#define Ponton2Section2		  	0x80
#define Ponton2Section3		  	0x08

#define Ponton1Section1		  	0x04
#define Ponton1Section2		  	0x02
#define Ponton1Section3		  	0x01


#pragma global_register sflag:20 gage:21 count10:22


//******************************************************************************
#define WaitDoneSPI() \
	while(!(SPSR & (1<<SPIF))); \
//******************************************************************************


//******************************************************************************
#define LoadPos() \
	PORTC &=~ 0x20; \
	PORTC |= 0x20; \
//******************************************************************************


//******************************************************************************
#define PumpSection1On() \
	PORTB &=~ 0x01;   \
	
#define PumpSection1Off() \
	PORTB |= 0x01;   \
//******************************************************************************	

//******************************************************************************	
#define PumpSection2On() \
	PORTB &=~ 0x02;   \
	
#define PumpSection2Off() \
	PORTB |= 0x02;   \
//******************************************************************************	

//******************************************************************************	
#define PumpSection3On() \
	PORTB &=~ 0x04;   \
	
#define PumpSection3Off() \
	PORTB |= 0x04;   \
//******************************************************************************	

//******************************************************************************	
#define TransmiteOn() \
	PORTC |= 0x10;   \

#define TransmiteOff() \
	PORTC &=~ 0x10;   \
//******************************************************************************	


unsigned char sflag, gage, count10, FrameLen;

//    
unsigned char SetIndicationCommand[] = {0x40,0x49,	  		// @I	  	
		 	  						   	0x30,0x30,0x30,		//   2,3,4
		 	  						   	0x30,0x30,0x30,		//   5,6,7
		 	  						   	0x30,0x30,0x30,		//   8,9,10
		 	  						   	0x30,0x30,0x30,		//   11,12,13
		 	  						   	0x30,0x30,0x30,		//   14,15,16
		 	  						   	0x30,0x30,0x30,		//   17,18,19
										0x00,0x00,			// [chk]=C9
										0x0D};   			// (cr)

unsigned char *Frame;		   		  //      
unsigned char bufer[60];	   		  //     


unsigned char CalcSummDCON(unsigned char *src, unsigned char len)
{
 register unsigned char sum, tm;

 sum = 0;
 do
 {
  sum += *src++;
  len--; 
 }while(len);

 tm = (sum >> 4) & 0x0F;
 if (9 < tm) tm += 0x37; else tm += 0x30; 
 *src++ = tm;

 tm = sum & 0x0F;
 if (9 < tm) tm += 0x37; else tm += 0x30; 
 *src++ = tm;
 
 return(sum);
}


//TIMER1 initialize - prescale:1024
// WGM: 0) Normal, TOP=0xFFFF
// desired value: 2Sec
// actual value:  2,000Sec (0,0%)
void timer1_init(void)
{
 TCCR1B = 0x00; //stop
 TCNT1H = 0x8F; //setup
 TCNT1L = 0x81;
 OCR1AH = 0x70;
 OCR1AL = 0x7F;
 OCR1BH = 0x70;
 OCR1BL = 0x7F;
 ICR1H  = 0x70;
 ICR1L  = 0x7F;
 TCCR1A = 0x00;
 TCCR1B = 0x05; //start Timer
}


#pragma interrupt_handler timer1_ovf_isr:9
void timer1_ovf_isr(void)
{
 //TIMER1 has overflowed
 TCNT1H = 0x8F; //reload counter high value
 TCNT1L = 0x81; //reload counter low value
 
 Frame = &SetIndicationCommand[0];		 	//      
 CalcSummDCON(Frame,20); 		   	   		//    
 sflag |= SendFrame;				   		//    
 FrameLen = 23;					   			//    
 TransmiteOn(); 				   			//   RS-485
 UDR = *Frame++;	   				   		//    
}


void port_init(void)
{
 PORTB = 0xFF;
 DDRB  = 0x2F;
 
 PORTC = 0x6F; 
 DDRC  = 0x30;
 
 PORTD = 0xFF;
 DDRD  = 0x02;
}


//Watchdog initialize
// prescale: 128K
void watchdog_init(void)
{
 WDR(); //this prevents a timout on enabling
 WDTCR = 0x0E; //WATCHDOG ENABLED - dont forget to issue WDRs
}


//TIMER0 initialize - prescale:1024
// desired value: 10mSec
// actual value:  9,931mSec (0,7%)
void timer0_init(void)
{
 TCCR0 = 0x00; //stop
 TCNT0 = 0x71; //set count
 TCCR0 = 0x05; //start timer
}


#pragma interrupt_handler timer0_ovf_isr:10
void timer0_ovf_isr(void)
{
 TCNT0 = 0x71; //reload counter value
 sflag |= int10flag;
 count10 ++;
 if(10 == count10) count10 = 0;
}


//SPI initialize
// clock rate: 3686399hz
void spi_init(void)
{
 SPCR = 0x5C; //setup SPI
 SPSR = 0x00; //setup SPI
}


//UART0 initialize
// desired baud rate: 9600
// actual: baud rate:9600 (0,0%)
// char size: 8 bit
// parity: Disabled
void uart0_init(void)
{
 UCSRB = 0x00; //disable while setting baud rate
 UCSRA = 0x00;
 UCSRC = BIT(URSEL) | 0x06;
 UBRRL = 0x5F; //set baud rate lo
 UBRRH = 0x00; //set baud rate hi
 UCSRB = 0xD8;
}


#pragma interrupt_handler uart0_rx_isr:12
void uart0_rx_isr(void)
{
 unsigned char RecivedByte;

 RecivedByte = UDR;  			//     
}


#pragma interrupt_handler uart0_tx_isr:14
void uart0_tx_isr(void)
{
 if(sflag & SendFrame)			//    
 {
  FrameLen--;					//   
  if(0 == FrameLen)	   			//    
  {
   sflag &=~ SendFrame;			//    
   TransmiteOff();				//    
   return;			   			//  
  }// if(0 == FrameLen)
  UDR = *Frame++;	   			//    
 }// if(uflag & SendFrame)
}


//call this routine to initialize all peripherals
void init_devices(void)
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
 port_init();
 watchdog_init();
 timer0_init();
 timer1_init();
 spi_init();
 uart0_init();

 MCUCR = 0x00;
 GICR  = 0x00;
 TIMSK = 0x05; //timer interrupt sources
 SEI(); //re-enable interrupts
 //all peripherals are now initialized
}

void main(void)
{
 unsigned char t,t2;
  
 sflag = 0;
 gage = 0;

 init_devices();
 
 do
 {
  if(sflag & int10flag)
  {
   sflag &=~ int10flag;	   		    //   
 
   WDR();

   if(1 == count10)					//   1    
   {
    gage = PIND;		   	   		  	//   
    if(!(gage & Section1Down)) 	  	//     1
    {
	 sflag |= AlarmSection1;	  	  	//      1
	 SetIndicationCommand[8] = 0x31; //     -1
	 if(!(gage & Section1Up))  	   	//        1
	 {
	  PumpSection1On()		  	  //     1
	 }//if(!(gage & Section1Up))
    }//if(!(gage & Section1Down))
    else 
    {
	 sflag &=~ AlarmSection1;	  //      1
	 SetIndicationCommand[8] = 0x30; //     -1
	 PumpSection1Off();		  	  //     1
    }// else

    if(!(gage & Section2Down)) 	  //     2
    {
	 sflag |= AlarmSection2;	  	  //      2
	 SetIndicationCommand[9] = 0x31; //     -2
	 if(!(gage & Section2Up))  	  //        2
	 {
	  PumpSection2On()		  	  //     2
	 }//if(!(gage & Section2Up))
    }//if(!(gage & Section2Down))
    else 
    {
	 sflag &=~ AlarmSection2;	  //      2
	 SetIndicationCommand[9] = 0x30; //     -2
	 PumpSection2Off();		  	  //     2
    }// else

    if(!(gage & Section3Down)) 	  //     3
    {
	 sflag |= AlarmSection3;	  	  //      3
	 SetIndicationCommand[11] = 0x31; //     -1
	 if(!(gage & Section3Up))  	  //        3
	 {
	  PumpSection3On()		  	  //     3
	 }//if(!(gage & Section3Up))
    }//if(!(gage & Section3Down))
    else 
    {
  	 sflag &=~ AlarmSection3;	  //      3
	 SetIndicationCommand[11] = 0x30; //     -1
	 PumpSection3Off();		  	  //     3
    }// else
   }// if(1 == count10)
   
   if(2==count10)		   			  //   2   
   {
    t = PINC & 0x0F;

//***************************************************************************************************************************************
	if(!(t & Ponton4Section3))			  //      4,  3
	{
	 SetIndicationCommand[19] = 0x31; //     -3
	}// if(!(t & Ponton4Section3))    
    else 
    {
	 SetIndicationCommand[19] = 0x30; //     -3
    }// else
	if(!(t & Ponton4Section2))			  //      4,  2
	{
	 SetIndicationCommand[18] = 0x31; //     -2
	}// if(!(t & Ponton4Section2))    
    else 
    {
	 SetIndicationCommand[18] = 0x30; //     -2
    }// else
	if(!(t & Ponton4Section1))			  //      4,  1
	{
	 SetIndicationCommand[17] = 0x31; //     -1
	}// if(!(t & Ponton4Section1))    
    else 
    {
	 SetIndicationCommand[17] = 0x30; //     -1
    }// else
//***************************************************************************************************************************************

//***************************************************************************************************************************************
	if(!(t & Ponton3Section1))			  //      3,  1
	{
	 SetIndicationCommand[14] = 0x31; //     -1
	}// if(!(t & Ponton3Section1))    
    else 
    {
	 SetIndicationCommand[14] = 0x30; //     H-1
    }// else

	LoadPos();						   //     
	SPDR = 0xFF;
    WaitDoneSPI(); 		  		   //      SPI
	t = SPDR; 
	if(!(t & Ponton3Section2))			  //      3,  2
	{
	 SetIndicationCommand[15] = 0x31; //     -2
	}// if(!(t & Ponton3Section2))    
    else 
    {
	 SetIndicationCommand[15] = 0x30; //     H-2
    }// else
	if(!(t & Ponton3Section3))			  //      3,  3
	{
	 SetIndicationCommand[16] = 0x31; //     -3
	}// if(!(t & Ponton3Section3))    
    else 
    {
	 SetIndicationCommand[16] = 0x30; //     H-3
    }// else
//***************************************************************************************************************************************


//***************************************************************************************************************************************
	if(!(t & Ponton2Section1))			  //      2,  1
	{
	 SetIndicationCommand[5] = 0x31; //     -1
	}// if(!(t & Ponton2Section1))    
    else 
    {
	 SetIndicationCommand[5] = 0x30; //     -1
    }// else
	
	if(!(t & Ponton2Section2))			  //      3,  2
	{
	 SetIndicationCommand[6] = 0x31; //     -2
	}// if(!(t & Ponton2Section2))    
    else 
    {
	 SetIndicationCommand[6] = 0x30; //     -2
    }// else

	if(!(t & Ponton2Section3))			  //      3,  3
	{
	 SetIndicationCommand[7] = 0x31; //     -3
	}// if(!(t & Ponton2Section3))    
    else 
    {
	 SetIndicationCommand[7] = 0x30; //     -3
    }// else
//***************************************************************************************************************************************

//***************************************************************************************************************************************
	if(!(t & Ponton1Section1))			  //      1,  1
	{
	 SetIndicationCommand[2] = 0x31; //     -1
	}// if(!(t & Ponton1Section1))    
    else 
    {
	 SetIndicationCommand[2] = 0x30; //     -1
    }// else
	
	if(!(t & Ponton1Section2))			  //      3,  2
	{
	 SetIndicationCommand[3] = 0x31; //     -2
	}// if(!(t & Ponton1Section2))    
    else 
    {
	 SetIndicationCommand[3] = 0x30; //     -2
    }// else

	if(!(t & Ponton1Section3))			  //      3,  3
	{
	 SetIndicationCommand[4] = 0x31; //     -3
	}// if(!(t & Ponton1Section3))    
    else 
    {
	 SetIndicationCommand[4] = 0x30; //     -3
    }// else
//***************************************************************************************************************************************
	
	
   }//if(2==count10)

   
  }//if(sflag & Int10Flag)
 } while(1);
}
